@@ -4,24 +4,20 @@ use strict;
 use warnings;
 use vars qw( $VERSION @EXPORT @EXPORT_OK @ISA $CurrentPackage @IncludeLibs $ScanFileRE );
 
-$VERSION   = '1.34';
+$VERSION   = '1.35';
 @EXPORT    = qw( scan_deps scan_deps_runtime );
 @EXPORT_OK = qw( scan_line scan_chunk add_deps scan_deps_runtime path_to_inc_name );
 
 use Config;
 require Exporter;
 our @ISA = qw(Exporter);
-use constant is_insensitive_fs => (
-    -s $0
-        and (-s lc($0) || -1) == (-s uc($0) || -1)
-        and (-s lc($0) || -1) == -s $0
-);
 
 use version;
 use File::Path ();
 use File::Temp ();
 use FileHandle;
 use Module::Metadata;
+use List::Util qw ( any first );
 
 # NOTE: Keep the following imports exactly as specified, even if the Module::ScanDeps source
 # doesn't reference some of them. See '"use lib" idioms' for the reason.
@@ -30,9 +26,13 @@ use File::Spec;
 use File::Spec::Functions;
 use File::Basename;
 
+use constant is_insensitive_fs => File::Spec->case_tolerant();
 
 $ScanFileRE = qr/(?:^|\\|\/)(?:[^.]*|.*\.(?i:p[ml]|t|al))$/;
 
+my %_glob_cache;
+my %_file_cache;
+my $_cached_inc = "";
 
 =head1 NAME
 
@@ -1156,22 +1156,24 @@ sub _add_info {
 
     # Avoid duplicates that can arise due to case differences that don't actually
     # matter on a case tolerant system
-    if (File::Spec->case_tolerant()) {
-        foreach my $key (keys %$rv) {
-            if (lc($key) eq lc($module)) {
-                $module = $key;
-                last;
-            }
+    if (is_insensitive_fs) {
+        if (!exists $rv->{$module}) {
+            my $lc_module  = lc $module;
+            my $key = first {lc($_) eq $lc_module} keys %$rv;
+            if (defined $key) {
+                $module = $key
+            };
         }
         if (defined($used_by)) {
             if (lc($used_by) eq lc($module)) {
                 $used_by = $module;
             } else {
-                foreach my $key (keys %$rv) {
-                    if (lc($key) eq lc($used_by)) {
-                        $used_by = $key;
-                        last;
-                    }
+                if (!exists $rv->{$used_by}) {
+                    my $lc_used_by = lc $used_by;
+                    my $key = first {lc($_) eq $lc_used_by} keys %$rv;
+                    if (defined $key) {
+                        $used_by = $key
+                    };
                 }
             }
         }
@@ -1184,14 +1186,22 @@ sub _add_info {
     };
 
     if (defined($used_by) and $used_by ne $module) {
-        push @{ $rv->{$module}{used_by} }, $used_by
-          if  ( (!File::Spec->case_tolerant() && !grep { $_ eq $used_by } @{ $rv->{$module}{used_by} })
-             or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($used_by) } @{ $rv->{$module}{used_by} }));
-
-        # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}
-        push @{ $rv->{$used_by}{uses} }, $module
-          if  ( (!File::Spec->case_tolerant() && !grep { $_ eq $module } @{ $rv->{$used_by}{uses} })
-             or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($module) } @{ $rv->{$used_by}{uses} }));
+        if (is_insensitive_fs) {
+            my $lc_used_by = lc $used_by;
+            my $lc_module  = lc $module;
+            push @{ $rv->{$module}{used_by} }, $used_by
+                if !any { lc($_) eq $lc_used_by } @{ $rv->{$module}{used_by} };
+            # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}
+            push @{ $rv->{$used_by}{uses} }, $module
+                if !any { lc($_) eq $lc_module } @{ $rv->{$used_by}{uses} };
+        }
+        else {
+            push @{ $rv->{$module}{used_by} }, $used_by
+                if !any { $_ eq $used_by } @{ $rv->{$module}{used_by} };
+            # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}
+            push @{ $rv->{$used_by}{uses} }, $module
+                if !any { $_ eq $module } @{ $rv->{$used_by}{uses} };
+        }
     }
 }
 
@@ -1254,12 +1264,31 @@ sub add_deps {
     return $rv;
 }
 
+# invalidate %_file_cache and %_glob_cache in case @INC changes
+sub _validate_cached_inc
+{
+    my $inc = join("\0", @INC, @IncludeLibs);
+    return if $inc eq $_cached_inc;
+
+    # blow away the caches
+    %_file_cache = ();
+    %_glob_cache = ();
+    $_cached_inc = $inc;
+}
+
 sub _find_in_inc {
     my $file = shift;
     return unless defined $file;
 
+    _validate_cached_inc();
+    my $cached_val = $_file_cache{$file};
+    return $cached_val if $cached_val;
+
     foreach my $dir (grep !/\bBSDPAN\b/, @INC, @IncludeLibs) {
-        return "$dir/$file" if -f "$dir/$file";
+        if (-f "$dir/$file") {
+            $_file_cache{$file} = "$dir/$file";
+            return "$dir/$file"
+        };
     }
 
     # absolute file names
@@ -1275,6 +1304,10 @@ sub _glob_in_inc {
 
     $subdir =~ s/\$CurrentPackage/$CurrentPackage/;
 
+    _validate_cached_inc();
+    my $cached_val = $_glob_cache{$subdir};
+    return @$cached_val if $cached_val;
+
     my @files;
     foreach my $inc (grep !/\bBSDPAN\b/, @INC, @IncludeLibs) {
         my $dir = "$inc/$subdir";
@@ -1296,6 +1329,8 @@ sub _glob_in_inc {
         );
     }
 
+    $_glob_cache{$subdir} = \@files;
+
     return @files;
 }
 
@@ -1672,7 +1707,7 @@ sub _merge_rv {
 
 sub _not_dup {
     my ($key, $rv1, $rv2) = @_;
-    if (File::Spec->case_tolerant()) {
+    if (is_insensitive_fs) {
         return lc(abs_path($rv1->{$key}{file})) ne lc(abs_path($rv2->{$key}{file}));
     }
     else {
